@@ -4,7 +4,7 @@ use warnings;
 use strict;
 
 use vars qw($VERSION);
-$VERSION = '0.14';
+$VERSION = '0.17';
 
 #----------------------------------------------------------------------------
 
@@ -64,7 +64,7 @@ my $no_index_1_2 = {
     }
 };
 my $no_index_1_1 = {
-    'map'       => { ':key'     => { name => \&word, list => { value => \&string } },
+    'map'       => { ':key'     => { name => \&keyword, list => { value => \&string } },
     }
 };
 
@@ -91,8 +91,8 @@ my %definitions = (
   'conflicts'           => $module_map2,
 
   'optional_features'   => {
-    list        => {
-        ':key'  => { name => \&word,
+    'map'       => {
+        ':key'  => { name => \&identifier,
             'map'   => { description        => { value => \&string },
                          requires_packages  => { value => \&string },
                          requires_os        => { value => \&string },
@@ -128,7 +128,7 @@ my %definitions = (
 
   # additional user defined key/value pairs
   # note we can only validate the key name, as the structure is user defined
-  ':key'        => { name => \&word },
+  ':key'        => { name => \&keyword },
 },
 
 '1.3' => {
@@ -152,8 +152,8 @@ my %definitions = (
   'conflicts'           => $module_map2,
 
   'optional_features'   => {
-    list        => {
-        ':key'  => { name => \&word,
+    'map'       => {
+        ':key'  => { name => \&identifier,
             'map'   => { description        => { value => \&string },
                          requires_packages  => { value => \&string },
                          requires_os        => { value => \&string },
@@ -189,7 +189,7 @@ my %definitions = (
 
   # additional user defined key/value pairs
   # note we can only validate the key name, as the structure is user defined
-  ':key'        => { name => \&word },
+  ':key'        => { name => \&keyword },
 },
 
 # v1.2 is misleading, it seems to assume that a number of fields where created
@@ -221,6 +221,22 @@ my %definitions = (
   'build_requires'      => $module_map1,
   'conflicts'           => $module_map2,
 
+  'optional_features'   => {
+    'map'       => {
+        ':key'  => { name => \&identifier,
+            'map'   => { description        => { value => \&string },
+                         requires_packages  => { value => \&string },
+                         requires_os        => { value => \&string },
+                         excludes_os        => { value => \&string },
+                         requires           => $module_map1,
+                         recommends         => $module_map1,
+                         build_requires     => $module_map1,
+                         conflicts          => $module_map2,
+            }
+        }
+     }
+  },
+
   'provides'    => {
     'map'       => { ':key' => { name  => \&module,
                                  'map' => { file    => { mandatory => 1, value => \&file },
@@ -238,7 +254,7 @@ my %definitions = (
 
   # additional user defined key/value pairs
   # note we can only validate the key name, as the structure is user defined
-  ':key'        => { name => \&word },
+  ':key'        => { name => \&keyword },
 },
 
 # note that the 1.1 spec doesn't specify optional or mandatory fields, what
@@ -263,7 +279,7 @@ my %definitions = (
 
   # additional user defined key/value pairs
   # note we can only validate the key name, as the structure is user defined
-  ':key'        => { name => \&word },
+  ':key'        => { name => \&keyword },
 },
 
 # note that the 1.0 spec doesn't specify optional or mandatory fields, what
@@ -285,7 +301,7 @@ my %definitions = (
 
   # additional user defined key/value pairs
   # note we can only validate the key name, as the structure is user defined
-  ':key'        => { name => \&word },
+  ':key'        => { name => \&keyword },
 },
 );
 
@@ -520,10 +536,21 @@ type, or 2 if a value is given but the license type is not a recommended one.
 Validates that the given key is in CamelCase, to indicate a user defined
 keyword.
 
-=item * word($self,$key,$value)
+=item * keyword($self,$key,$value)
 
 Validates that key is in an acceptable format for the META.yml specification,
-i.e. any in the character class [-_a-z].
+i.e. any in the character class [-_a-z]. 
+
+For user defined keys, although not explicitly stated in the specifications 
+(v1.0 - v1.4), the convention is to precede the key with a pattern matching 
+qr{\Ax_}i. Following this any character from the character class [-_a-zA-Z]
+can be used. This clarification has been added to v2.0 of the specification.
+
+=item * identifier($self,$key,$value)
+
+Validates that key is in an acceptable format for the META.yml specification,
+for an identifier, i.e. any that matches the regular expression
+qr/[a-z][a-z_]/i. 
 
 =item * module($self,$key,$value)
 
@@ -543,20 +570,20 @@ sub header {
     return 0;
 }
 
-#my $protocol = qr(?:http|https|ftp|afs|news|nntp|mid|cid|mailto|wais|prospero|telnet|gopher);
-my $protocol = qr(?:ftp|http|https|git);
-my $badproto = qr(\w+)://;
-my $proto    = qr$protocol://(?:[\w]+:\w+@)?;
-my $atom     = qr[a-z\d]i;
-my $domain   = qr((($atom(($atom|-)*$atom)?)\.)*([a-zA-Z](($atom|-)*$atom)?));
-my $ip       = qr((\d+)(\.(\d+)){3})(:(\d+))?;
-my $enc      = qr%[a-fA-F\d]{2};
-my $legal1   = qr[a-zA-Z\d\$\-_.+!*'(),]; #' - this comment is to avoid syntax highlighting issues
-my $legal2   = qr[;:@&=];
-my $legal3   = qr((($legal1|$enc)|$legal2)*);
-my $path     = qr\/$legal3(\/$legal3)*;
-my $query    = qr\?$legal3;
-my $urlregex = qr(($proto)?($domain|$ip)(($path)?($query)?)?);
+#my $protocol = qr!(?:http|https|ftp|afs|news|nntp|mid|cid|mailto|wais|prospero|telnet|gopher)!;
+my $protocol = qr!(?:ftp|http|https|git)!;
+my $badproto = qr!(\w+)://!;
+my $proto    = qr!$protocol://(?:[\w]+:\w+@)?!;
+my $atom     = qr![a-z\d]!i;
+my $domain   = qr!((($atom(($atom|-)*$atom)?)\.)*([a-zA-Z](($atom|-)*$atom)?))!;
+my $ip       = qr!((\d+)(\.(\d+)){3})(:(\d+))?!;
+my $enc      = qr!%[a-fA-F\d]{2}!;
+my $legal1   = qr|[a-zA-Z\d\$\-_.+!*\'(),#]|;
+my $legal2   = qr![;:@&=]!;
+my $legal3   = qr!((($legal1|$enc)|$legal2)*)!;
+my $path     = qr!\/$legal3(\/$legal3)*!;
+my $query    = qr!\?$legal3!;
+my $urlregex = qr!(($proto)?($domain|$ip)(($path)?($query)?)?)!;
 
 sub url {
     my ($self,$key,$value) = @_;
@@ -691,10 +718,11 @@ sub resource {
     return 0;
 }
 
-sub word {
+sub keyword {
     my ($self,$key) = @_;
     if(defined $key) {
-        return 1    if($key && $key =~ /^([-_a-z]+)$/);
+        return 1    if($key && $key =~ /^([-_a-z]+)$/);     # spec defined
+        return 1    if($key && $key =~ /^x_([-_a-z]+)$/i);  # user defined
     } else {
         $key = '<undef>';
     }
@@ -702,6 +730,17 @@ sub word {
     return 0;
 }
 
+sub identifier {
+    my ($self,$key) = @_;
+    if(defined $key) {
+        return 1    if($key && $key =~ /^([a-z][_a-z]+)$/i);    # spec 2.0 defined
+    } else {
+        $key = '<undef>';
+    }
+    $self->_error( "Key '$key' is not a legal identifier." );
+    return 0;
+}
+
 sub module {
     my ($self,$key) = @_;
     if(defined $key) {
@@ -756,7 +795,7 @@ for Miss Barbell Productions, L<http://www.missbarbell.co.uk>
 
 =head1 COPYRIGHT AND LICENSE
 
-  Copyright (C) 2007,2008 Barbie for Miss Barbell Productions
+  Copyright (C) 2007-2010 Barbie for Miss Barbell Productions
 
   This module is free software; you can redistribute it and/or
   modify it under the same terms as Perl itself.
